'''
Created on 1.8.2012

@author: xaralis
'''
from django.core.management.base import NoArgsCommand
from django.db import connection
from django.db.utils import IntegrityError

try:
    import tagging
    import taggit
except ImportError, e:

    class Command(NoArgsCommand):
        def handle_noargs(self, **kwargs):
            print "Missing tag-library, aborting: %s" % e

else:
    from tagging.models import Tag as OldTag
    from tagging.models import TaggedItem as OldTaggedItem
    from ella_taggit.models import PublishableTag as NewTag
    from ella_taggit.models import PublishableTaggedItem as NewTaggedItem

    class Command(NoArgsCommand):
        def handle_noargs(self, **kwargs):
            NewTag.objects.all().delete()
            NewTaggedItem.objects.all().delete()
            self.delete_unused_tags()
            self.set_tags()
            self.set_taggeditems()
            self.drop_old_tables()

        def get_used_tags(self):
            return tuple(set(t[0] for t in
                    OldTaggedItem.objects.only('tag').values_list('tag')))

        def delete_unused_tags(self):
            used_tags = self.get_used_tags()

            for t in tuple(OldTag.objects.all()):
                if t.id not in used_tags:
                    t.delete()
                    print "Deleted: %s (%s)" % (t.id, t.name)

        def set_tags(self):
            has_dupes = {}
            for old_tag in OldTag.objects.all():
                new_tag = NewTag(id=old_tag.id, name=old_tag.name)
                slug = new_tag.slugify(old_tag.name)
                try:
                    new_tag = NewTag.objects.get(slug=slug)
                    has_dupes.setdefault(new_tag.id, []).append(old_tag.id)
                    print 'ERROR: %s (%i,%i) already exists' % (slug, new_tag.id, old_tag.id)
                except NewTag.DoesNotExist:
                    new_tag.slug = slug
                    new_tag.save()
            self.dupemap = {}
            for first, later in has_dupes.items():
                for dupe in later:
                    self.dupemap[dupe] = first
            print '%i dupes' % len(self.dupemap)

        def set_taggeditems(self):
            objects = OldTaggedItem.objects.all()
            print 'Assigning tags to %i Publishables' % objects.count()
            for old_tagitem in objects:
                old_id = old_tagitem.tag_id
                new_id = self.dupemap.get(old_id, old_id)
                new_tags = NewTag.objects.filter(id=new_id)
                if new_tags:
                    new_tag = new_tags[0]
                    try:
                        new_tagitem = NewTaggedItem(
                                id=old_tagitem.id,
                                tag=new_tag,
                                content_object_id=old_tagitem.object_id)
                        new_tagitem.save()
                    except IntegrityError:
                        print 'Cannot assingn tag %s to publishable %s: ' \
                              'Publishable object not found.' % (
                                  new_tag, old_tagitem.object_id)

        def drop_old_tables(self):
            print "Dropping django-tagging tables."
            cursor = connection.cursor()
            cursor.execute('DROP TABLE tagging_taggeditem;')
            cursor.execute('DROP TABLE tagging_tag;')



'''
Created on 1.8.2012

@author: xaralis
'''
from django.template.defaultfilters import slugify
from django.views.generic import ListView
from django.conf import settings

from ella_taggit.models import PublishableTag, tag_list, publishables_with_tag
from ella.core.cache.utils import get_cached_object_or_404


class TaggedPublishablesView(ListView):
    context_object_name = 'listings'
    paginate_by = getattr(settings, 'TAG_LISTINGS_PAGINATE_BY', 10)
    relation_occ_threshold = getattr(settings, 'TAG_RELATION_OCCURENCE_THRESHOLD', None)
    relation_count_limit = getattr(settings, 'TAG_RELATION_COUNT_LIMIT', 5)

    def get_queryset(self, **kwargs):
        self.tag = get_cached_object_or_404(PublishableTag, slug=self.kwargs['tag'])
        return publishables_with_tag(self.tag, filters=kwargs)

    def get_template_names(self):
        return (
            'page/tagging/%s/listing.html' % slugify(self.kwargs['tag']),
            'page/tagging/listing.html',
        )

    def paginate_queryset(self, queryset, page_size):
        """
        Ella uses it's own pagination style. If you want django's style,
        delete this function.
        """
        paginator = self.get_paginator(queryset,
                                       page_size,
                                       allow_empty_first_page=self.get_allow_empty())

        if 'p' in self.request.GET and self.request.GET['p'].isdigit():
            page_no = int(self.request.GET['p'])
        else:
            page_no = 1

        page = paginator.page(page_no)
        return (paginator, page, page.object_list, page.has_other_pages())

    def get_context_data(self, **kwargs):
        context = super(TaggedPublishablesView, self).get_context_data(**kwargs)
        if context['is_paginated']:
            context['page'] = context['page_obj']
            context['results_per_page'] = self.paginate_by

        context['tag'] = self.tag
        context['related_tags'] = tag_list(self.object_list,
                                           threshold=self.relation_occ_threshold,
                                           count=self.relation_count_limit,
                                           omit=self.tag)
        return context



from os.path import dirname, join, normpath, pardir

FILE_ROOT = normpath(join(dirname(__file__), pardir))

USE_I18N = True

MEDIA_ROOT = join(FILE_ROOT, 'static')


# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'test_ella.template_loader.load_template_source',
)


MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
)

ROOT_URLCONF = 'test_ella_taggit.urls'

TEMPLATE_DIRS = (
    # Put strings here, like "/home/html/django_templates" or "C:/www/django/templates".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
    join(FILE_ROOT, 'templates'),

)

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.redirects',
    'django.contrib.admin',
    'django.contrib.comments',

    'ella.core',
    'ella.photos',
    'ella.articles',
    'ella_taggit',
)

TAG_LISTINGS_PAGINATE_BY = 1

RELATED_FINDERS = {
    'default': (
        'ella.core.related_finders.directly_related',
        'ella_taggit.related_finders.related_by_tags',
        'ella.core.related_finders.related_by_category',
    )
}




#!/usr/bin/env python

import os
from os.path import join, pardir, abspath, dirname, split
import sys

from django.core.management import execute_from_command_line


# fix PYTHONPATH and DJANGO_SETTINGS for us
# django settings module
DJANGO_SETTINGS_MODULE = '%s.%s' % (split(abspath(dirname(__file__)))[1], 'settings')
# pythonpath dirs
PYTHONPATH = [
    abspath(join( dirname(__file__), pardir, pardir)),
    abspath(join( dirname(__file__), pardir)),
]

# inject few paths to pythonpath
for p in PYTHONPATH:
    if p not in sys.path:
        sys.path.insert(0, p)

# django needs this env variable
os.environ['DJANGO_SETTINGS_MODULE'] = DJANGO_SETTINGS_MODULE


if __name__ == "__main__":
    execute_from_command_line()




'''
Created on 1.8.2012

@author: xaralis
'''
import hashlib

import cPickle
from collections import defaultdict
from datetime import datetime
import operator

from django.db import models
from django.utils.translation import ugettext_lazy as _

from ella.core.models import Publishable
from ella.core.cache.utils import cache_this

from taggit.managers import TaggableManager, _TaggableManager
from taggit.models import ItemBase, TagBase
from taggit.utils import require_instance_manager


def get_tag_list_key(object_list, threshold, count, omit):
    return 'ella_taggit.models.tag_list:%s:%d:%d:%d' % (
        hashlib.md5(','.join([str(o.pk) for o in object_list])).hexdigest(),
        threshold or 0,
        count or 0,
        omit.pk)


@cache_this(get_tag_list_key)
def tag_list(object_list, threshold=None, count=None, omit=None):
    """
    Returns all tags appearing for any object in `object_list` ordered
    by their occurence.

    Optionally, max `count` can be specified and also a `threshold` which have
    to be met if a tag would be in the result list.

    Moreover, if you specify `omit` argument by supplying `PublishableTag`
    instance, this tag (if present in results) will be omitted.
    """
    occ_map = defaultdict(int)
    tags = PublishableTag.objects.filter(
        tagged_items__content_object__in=object_list)

    if omit is not None:
        tags = tags.exclude(pk=omit.pk)

    for t in tags:
        occ_map[t] += 1

    if threshold is not None:
        occ_map = dict((tag, occ)
                       for tag, occ in occ_map.iteritems()
                       if occ >= threshold)

    tag_list = map(operator.itemgetter(0),
                   reversed(sorted(occ_map.items(), key=lambda i: i[1])))

    if count is not None:
        tag_list = tag_list[:count]

    return tag_list


def get_publishables_with_tag_key(tag, **kwargs):
    tpl = u'ella_taggit.models.pub_w_t:%s:' + hashlib.md5(cPickle.dumps(sorted(kwargs.iteritems()))).hexdigest()

    if hasattr(tag, '__iter__'):
        return tpl % u';'.join([t.slug for t in sorted(tag, key=lambda i: i.id)])

    return tpl % tag.slug


def _publishables_with_tag(tag=(), filters={}, excludes={}):
    now = datetime.now()

    if not hasattr(tag, '__iter__'):
        tag = [tag]

    qset = Publishable.objects.filter(
        models.Q(publish_to__isnull=True) | models.Q(publish_to__gt=now),
        publish_from__lt=now,
        published=True,
        tags__in=tag,
        **filters
    )

    if excludes:
        qset = qset.exclude(**excludes)

    return qset.distinct().order_by('-publish_from')


def publishables_with_tag(tag=(), filters={}, excludes={}, cache=True):
    fn = _publishables_with_tag

    if cache:
        fn = cache_this(get_publishables_with_tag_key)(fn)

    return fn(tag, filters=filters, excludes=excludes)


class PublishableTag(TagBase):
    description = models.TextField(verbose_name=_('Description'), blank=True,
                                   null=True)

    class Meta:
        verbose_name = _('Publishable tag')
        verbose_name_plural = _('Publishable tags')

    @models.permalink
    def get_absolute_url(self):
        return ('tag', [self.slug])


class PublishableTaggedItem(ItemBase):
    tag = models.ForeignKey(PublishableTag, related_name="tagged_items")
    content_object = models.ForeignKey('core.Publishable')

    class Meta:
        verbose_name = _('Tagged item')
        verbose_name_plural = _('Tagged items')

    @classmethod
    def tags_for(cls, model, instance=None):
        if instance is not None:
            return cls.tag_model().objects.filter(**{
                '%s__content_object' % cls.tag_relname(): instance
            })
        return cls.tag_model().objects.filter(**{
            '%s__content_object__isnull' % cls.tag_relname(): False
        }).distinct()


# Since django-taggit doesn't support excludes attibute in `similar_objects`
# method so far, provide an alternative which is also a little faster.
class _PublishableTaggableManager(_TaggableManager):
    @require_instance_manager
    def similar_objects(self, num=None, filters={}, excludes={}):
        lookup_kwargs = self._lookup_kwargs()

        qs = self.through.objects.all()

        if excludes is not None:
            lookup_kwargs.update(**excludes)

        qs = qs.annotate(n=models.Count('pk'))
        qs = qs.exclude(**lookup_kwargs)

        subq = self.all()

        qs = qs.filter(tag__in=list(subq))
        qs = qs.order_by('-n')

        if filters is not None:
            qs = qs.filter(**filters)

        if num is not None:
            qs = qs[:num]

        return list(set(pti.content_object for pti in qs))


class PublishableTaggableManager(TaggableManager):
    def __get__(self, instance, model):
        if instance is not None and instance.pk is None:
            raise ValueError("%s objects need to have a primary key value "
                "before you can access their tags." % model.__name__)
        manager = _PublishableTaggableManager(
            through=self.through, model=model, instance=instance
        )
        return manager


from south.modelsinspector import add_introspection_rules
add_introspection_rules([], ["ella_taggit.models.PublishableTaggableManager"])

# Patch Publishable class to add `tags` attribute
Publishable.add_to_class('tags', PublishableTaggableManager(through=PublishableTaggedItem))



# -*- coding: utf-8 -*-
from datetime import datetime, timedelta

from django.core.urlresolvers import reverse
from django.test import TestCase

from nose import tools

from ella.articles.models import Article
from ella.core.models.publishable import Publishable

from test_ella.test_core import create_basic_categories, create_and_place_a_publishable
from test_ella import template_loader

from ella_taggit.models import PublishableTag as Tag, PublishableTaggedItem as TaggedItem, \
    publishables_with_tag, tag_list


class TestTaggingViews(TestCase):
    def setUp(self):
        super(TestTaggingViews, self).setUp()
        create_basic_categories(self)
        create_and_place_a_publishable(self)

    def tearDown(self):
        super(TestTaggingViews, self).tearDown()
        template_loader.templates = {}

    def test_publishables_with_tag_returns_tagged_publishable(self):
        self.only_publishable.tags.set('tag1', 'tag2')
        tools.assert_equals(2, TaggedItem.objects.count())

        t = Tag.objects.get(name='tag1')
        tools.assert_equals([self.publishable], [p.target for p in publishables_with_tag(t)])

    def test_publishables_with_tag_doesnt_return_tagged_publishable_with_future_placement(self):
        self.only_publishable.tags.set('tag1', 'tag2')
        tools.assert_equals(2, TaggedItem.objects.count())

        self.publishable.publish_from = datetime.now() + timedelta(days=2)
        self.publishable.save()

        t = Tag.objects.get(name='tag1')
        tools.assert_equals([], list(publishables_with_tag(t)))

    def test_publishables_with_tag_returns_no_objects_when_none_tagged(self):
        t = Tag.objects.create(name='tag1')
        tools.assert_equals([], list(publishables_with_tag(t)))

    def test_tagged_publishables_view(self):
        self.only_publishable.tags.set('tag1', 'tag2')
        tools.assert_equals(2, TaggedItem.objects.count())

        t = Tag.objects.get(name='tag1')
        url = reverse('tag', kwargs={'tag': 'tag1'})
        template_loader.templates['page/tagging/listing.html'] = ''
        response = self.client.get(url)

        tools.assert_equals([self.publishable], [p.target for p in response.context['object_list']])
        tools.assert_equals(t, response.context['tag'])


class TestTagList(TestCase):
    def setUp(self):
        super(TestTagList, self).setUp()
        create_basic_categories(self)
        create_and_place_a_publishable(self)

    def test_tag_list_omits_if_requested(self):
        self.only_publishable.tags.set('tag1', 'tag2', 'tag3')
        t1 = Tag.objects.get(name='tag1')
        t2 = Tag.objects.get(name='tag2')
        t3 = Tag.objects.get(name='tag3')

        tools.assert_equals(list(tag_list([self.only_publishable])), [t3, t2, t1])
        tools.assert_equals(list(tag_list([self.only_publishable], omit=t1)), [t3, t2])

    def test_tag_list_limits_result_count(self):
        self.only_publishable.tags.set('tag1', 'tag2', 'tag3')
        tools.assert_equals(len(tag_list([self.only_publishable])), 3)
        tools.assert_equals(len(tag_list([self.only_publishable], count=2)), 2)

    def test_tag_list_adheres_to_threshold(self):
        self.pub = Article.objects.create(
            title=u'taglist2',
            slug=u'taglist2',
            description=u'taglist',
            category=self.category_nested,
            publish_from=datetime(2008, 1, 1),
            published=True
        )
        self.only_pub = self.pub.publishable_ptr

        self.only_publishable.tags.set('tag1', 'tag2', 'tag3')
        self.only_pub.tags.set('tag1', 'tag2')

        t1 = Tag.objects.get(name='tag1')
        t2 = Tag.objects.get(name='tag2')

        tools.assert_equals(tag_list([self.only_publishable, self.only_pub], threshold=2), [t2, t1])

    def test_tag_list_returns_items_in_order_of_occurence_count(self):
        self.pub = Article.objects.create(
            title=u'taglist2',
            slug=u'taglist2',
            description=u'taglist',
            category=self.category_nested,
            publish_from=datetime(2008, 1, 1),
            published=True
        )
        self.only_pub = self.pub.publishable_ptr

        self.pub2 = Article.objects.create(
            title=u'taglist3',
            slug=u'taglist3',
            description=u'taglist',
            category=self.category_nested,
            publish_from=datetime(2008, 5, 1),
            published=True
        )
        self.only_pub2 = self.pub2.publishable_ptr

        self.only_publishable.tags.set('tag1', 'tag2', 'tag3')
        self.only_pub.tags.set('tag1', 'tag2')
        self.only_pub2.tags.set('tag1')

        t1 = Tag.objects.get(name='tag1')
        t2 = Tag.objects.get(name='tag2')
        t3 = Tag.objects.get(name='tag3')

        tools.assert_equals(tag_list([self.only_publishable,
                                      self.only_pub,
                                      self.only_pub2]),
                                      [t1, t2, t3])



'''
Created on 1.8.2012

@author: xaralis
'''
from django.conf import settings
from django.conf.urls.defaults import patterns, url

from ella_taggit.views import TaggedPublishablesView


try:
    if getattr(settings, 'TAGGIT_CUSTOM_VIEWS', False):
        views = settings.TAGGIT_VIEWS
        temp = __import__(views, globals(), locals(), ['TaggedPublishablesView'])
        TaggedPublishablesView = temp.TaggedPublishablesView
except ImportError:
    pass

urlpatterns = patterns('',
    url(r'^(?P<tag>[-\w\s]+)/', TaggedPublishablesView.as_view(), name="tag"),
)



from nose import tools

from ella.core.models import Related, Publishable

from ella_taggit.models import PublishableTag as Tag, PublishableTaggedItem as TaggedItem

from test_ella.test_core.test_related import GetRelatedTestCase


class TestGetRelatedWithtagging(GetRelatedTestCase):
    def setUp(self):
        super(TestGetRelatedWithtagging, self).setUp()
        self.Tag = Tag
        self.TaggedItem = TaggedItem

    def test_returns_object_with_similar_tags(self):
        self.only_publishable.tags.set('tag1', 'tag2')
        p = Publishable.objects.get(pk=self.publishables[0].pk)
        p.tags.set('tag1', 'tag2')
        tools.assert_equals([p], Related.objects.get_related_for_object(self.publishable, 1))





import ella_newman as newman
from ella_newman.conf import newman_settings

from django.utils.translation import ugettext_lazy as _

from ella_taggit.models import PublishableTag, PublishableTaggedItem


class TaggingInlineAdmin(newman.NewmanTabularInline):
    model = PublishableTaggedItem
    max_num = newman_settings.MAX_TAGS_INLINE
    suggest_fields = {'tag': ('name',)}
#    formset = TaggingInlineFormset


class TagAdmin(newman.NewmanModelAdmin):
    list_display = ('name', 'slug')
    search_fields = ('name', 'slug')


class TaggedItemAdmin(newman.NewmanModelAdmin):
    list_display = ('content_object', 'tag',)
    search_fields = ('tag',)
    suggest_fields = {'tag': ('name',)}

newman.site.register(PublishableTag, TagAdmin)
newman.site.register(PublishableTaggedItem, TaggedItemAdmin)
newman.site.append_inline(newman.site.MODELS_ALL_PUBLISHABLE, TaggingInlineAdmin)

# some translations for newman:

app = _('Tagging')
sg, pl = _('tag'), _('tags')



ADMINS = ()
MANAGERS = ADMINS

DEBUG = False
TEMPLATE_DEBUG = DEBUG
DISABLE_CACHE_TEMPLATE = DEBUG

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}

TIME_ZONE = 'Europe/Prague'

LANGUAGE_CODE = 'en-us'

SITE_ID = 1

# Make this unique, and don't share it with anybody.
SECRET_KEY = '88b-01asdlk329478(&^#$Q#l234%&:ASDk@#$'

# we want to reset whole cache in test
# until we do that, don't use cache
CACHE_BACKEND = 'dummy://'




"""
In this package, You can find test environment for Ella taggit unittest project.
As only true unittest and "unittest" (test testing programming unit, but using
database et al) are there, there is not much setup around.
"""
import os

test_runner = None
old_config = None

os.environ['DJANGO_SETTINGS_MODULE'] = 'test_ella_taggit.settings'


def setup():
    global test_runner
    global old_config
    from django.test.simple import DjangoTestSuiteRunner
    test_runner = DjangoTestSuiteRunner()
    test_runner.setup_test_environment()
    old_config = test_runner.setup_databases()


def teardown():
    test_runner.teardown_databases(old_config)
    test_runner.teardown_test_environment()




"""
Settings package is acting exactly like settings module in standard django projects.
However, settings combines two distinct things:
  (1) General project configuration, which is property of the project
    (like which application to use, URL configuration, authentication backends...)
  (2) Machine-specific environment configuration (database to use, cache URL, ...)

Thus, we're changing module into package:
  * base.py contains (1), so no adjustments there should be needed to make project
    on your machine
  * config.py contains (2) with sensible default values that should make project
    runnable on most expected machines
  * local.py contains (2) for your specific machine. File your defaults there. 
"""

from test_ella_taggit.settings.base import *
from test_ella_taggit.settings.config import *

try:
    from test_ella_taggit.settings.local import *
except ImportError:
    pass



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'PublishableTag'
        db.create_table('ella_taggit_publishabletag', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(unique=True, max_length=100)),
            ('slug', self.gf('django.db.models.fields.SlugField')(unique=True, max_length=100)),
            ('description', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
        ))
        db.send_create_signal('ella_taggit', ['PublishableTag'])

        # Adding model 'PublishableTaggedItem'
        db.create_table('ella_taggit_publishabletaggeditem', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('tag', self.gf('django.db.models.fields.related.ForeignKey')(related_name='ella_taggit_publishabletaggeditem_items', to=orm['ella_taggit.PublishableTag'])),
            ('content_object', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['core.Publishable'])),
        ))
        db.send_create_signal('ella_taggit', ['PublishableTaggedItem'])


    def backwards(self, orm):
        # Deleting model 'PublishableTag'
        db.delete_table('ella_taggit_publishabletag')

        # Deleting model 'PublishableTaggedItem'
        db.delete_table('ella_taggit_publishabletaggeditem')


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'core.author': {
            'Meta': {'object_name': 'Author'},
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '255'}),
            'text': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']", 'null': 'True', 'blank': 'True'})
        },
        'core.category': {
            'Meta': {'unique_together': "(('site', 'tree_path'),)", 'object_name': 'Category'},
            'app_data': ('jsonfield.fields.JSONField', [], {'default': "'{}'"}),
            'content': ('django.db.models.fields.TextField', [], {'default': "''", 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['sites.Site']"}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '255'}),
            'template': ('django.db.models.fields.CharField', [], {'default': "'category.html'", 'max_length': '100'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'tree_parent': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.Category']", 'null': 'True', 'blank': 'True'}),
            'tree_path': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        'core.publishable': {
            'Meta': {'object_name': 'Publishable'},
            'announced': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'app_data': ('jsonfield.fields.JSONField', [], {'default': "'{}'"}),
            'authors': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['core.Author']", 'symmetrical': 'False'}),
            'category': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.Category']"}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_updated': ('django.db.models.fields.DateTimeField', [], {'blank': 'True'}),
            'photo': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['photos.Photo']", 'null': 'True', 'on_delete': 'models.SET_NULL', 'blank': 'True'}),
            'publish_from': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(3000, 1, 1, 0, 0, 0, 2)', 'db_index': 'True'}),
            'publish_to': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'published': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '255'}),
            'source': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.Source']", 'null': 'True', 'blank': 'True'}),
            'static': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        'core.source': {
            'Meta': {'object_name': 'Source'},
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'url': ('django.db.models.fields.URLField', [], {'max_length': '200', 'blank': 'True'})
        },
        'ella_taggit.publishabletag': {
            'Meta': {'object_name': 'PublishableTag'},
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '100'})
        },
        'ella_taggit.publishabletaggeditem': {
            'Meta': {'object_name': 'PublishableTaggedItem'},
            'content_object': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.Publishable']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'tag': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'ella_taggit_publishabletaggeditem_items'", 'to': "orm['ella_taggit.PublishableTag']"})
        },
        'photos.photo': {
            'Meta': {'object_name': 'Photo'},
            'app_data': ('jsonfield.fields.JSONField', [], {'default': "'{}'", 'blank': 'True'}),
            'authors': ('django.db.models.fields.related.ManyToManyField', [], {'related_name': "'photo_set'", 'symmetrical': 'False', 'to': "orm['core.Author']"}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'height': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('django.db.models.fields.files.ImageField', [], {'max_length': '255'}),
            'important_bottom': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'important_left': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'important_right': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'important_top': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '255'}),
            'source': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.Source']", 'null': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'width': ('django.db.models.fields.PositiveIntegerField', [], {})
        },
        'sites.site': {
            'Meta': {'ordering': "('domain',)", 'object_name': 'Site', 'db_table': "'django_site'"},
            'domain': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        }
    }

    complete_apps = ['ella_taggit']


#!/usr/bin/env python

'''
simple shortcut for running nosetests via python
replacement for *.bat or *.sh wrappers
'''

import sys
from os.path import abspath, dirname

import nose


def run_all(argv=None):
    sys.exitfunc = lambda msg = 'Process shutting down...': sys.stderr.write(msg + '\n')

    if argv is None:
        argv = [
            'nosetests',
            '--with-coverage', '--cover-package=ella_taggit', '--cover-erase',
            '--nocapture', '--nologcapture',
            '--verbose',
        ]

    nose.run_exit(
        argv=argv,
        defaultTest=abspath(dirname(__file__)),
    )

if __name__ == '__main__':
    run_all(sys.argv)




'''
Created on 2.8.2012

@author: xaralis
'''
from django import template

from ella.core.models.publishable import Publishable

register = template.Library()


class MostCommonTagsNode(template.Node):
    def __init__(self, count, varname):
        self.count = count
        self.varname = varname

    def render(self, context):
        context[self.varname] = Publishable.tags.most_common()[:self.count]
        return ''


@register.tag
def get_most_common_tags(parser, token):
    try:
        tag_name, count, fill, varname = token.split_contents()
    except ValueError:
        try:
            tag_name, fill, varname = token.split_contents()
            count = 10
        except ValueError:
            raise template.TemplateSyntaxError('`get_most_common_tags` tag '
                                               'must follow form '
                                               '{% get_most_common_tags [COUNT] as [VARNAME] %}')

    return MostCommonTagsNode(count, varname)


class TagsForObjectNode(template.Node):
    def __init__(self, obj, varname):
        self.obj = template.Variable(obj)
        self.varname = varname

    def render(self, context):
        context[self.varname] = self.obj.resolve(context).tags.all()
        return ''


@register.tag
def tags_for_object(parser, token):
    try:
        tag_name, obj, fill, varname = token.split_contents()
        return TagsForObjectNode(obj, varname)
    except ValueError:
        raise template.TemplateSyntaxError('`tags_for_object` tag requires '
                                           'exactly 3 argument')



# -*- coding: utf-8 -*-
from datetime import datetime

from django.conf import settings
from django.core.urlresolvers import reverse
from django.test.client import RequestFactory
from django.test import TestCase

from nose import tools

from ella.articles.models import Article
from ella.core.models import Publishable

from test_ella.test_core import create_basic_categories, create_and_place_a_publishable

from ella_taggit.models import PublishableTag as Tag
from ella_taggit.views import TaggedPublishablesView


class TestTaggingPagination(TestCase):
    """Tests depends on setttings TAG_LISTINGS_PAGINATE_BY = 1"""
    def setUp(self):
        super(TestTaggingPagination, self).setUp()

        settings.TAG_LISTINGS_PAGINATE_BY = 1
        create_basic_categories(self)
        create_and_place_a_publishable(self)
        self.factory = RequestFactory()

        self.pub = Article.objects.create(
            title=u'Pagination',
            slug=u'pagination',
            description=u'Testing paggination',
            category=self.category_nested,
            publish_from=datetime(2008, 1, 1),
            published=True
        )

        self.only_pub = Publishable.objects.get(pk=self.pub.pk)
        self.only_pub.tags.set('tag1')
        self.only_publishable.tags.set('tag1')
        self.tag = Tag.objects.get(name='tag1')

    def tearDown(self):
        super(TestTaggingPagination, self).tearDown()
        del settings.TAG_LISTINGS_PAGINATE_BY

    def test_tagged_paggination_view_without_page_parameter(self):
        url = reverse('tag', kwargs={'tag': 'tag1'})
        request = self.factory.get(url)
        tpv = TaggedPublishablesView.as_view()
        response = tpv(request, tag='tag1')
        tools.assert_true(200, response.status_code)
        tools.assert_true(response.context_data['is_paginated'])
        tools.assert_equals(1, response.context_data['results_per_page'])
        tools.assert_equals(self.tag, response.context_data['tag'])
        tools.assert_equals(1, len(response.context_data['object_list']))
        tools.assert_equals(self.only_publishable,
                           response.context_data['object_list'][0])

    def test_tagged_paggination_view_with_page1_parameter(self):
        url = reverse('tag', kwargs={'tag': 'tag1'})
        request = self.factory.get(url, {'p': 1})
        tpv = TaggedPublishablesView.as_view()
        response = tpv(request, tag='tag1')
        tools.assert_true(200, response.status_code)
        tools.assert_true(response.context_data['is_paginated'])
        tools.assert_equals(1, response.context_data['results_per_page'])
        tools.assert_equals(self.tag, response.context_data['tag'])
        tools.assert_equals(1, len(response.context_data['object_list']))
        tools.assert_equals(self.only_publishable,
                           response.context_data['object_list'][0])

    def test_tagged_paggination_view_with_page2_parameter(self):
        url = reverse('tag', kwargs={'tag': 'tag1'})
        request = self.factory.get(url, {'p': 2})
        tpv = TaggedPublishablesView.as_view()
        response = tpv(request, tag='tag1')
        tools.assert_true(200, response.status_code)
        tools.assert_true(response.context_data['is_paginated'])
        tools.assert_equals(1, response.context_data['results_per_page'])
        tools.assert_equals(self.tag, response.context_data['tag'])
        tools.assert_equals(1, len(response.context_data['object_list']))
        tools.assert_equals(self.only_pub,
                           response.context_data['object_list'][0])



from setuptools import setup, find_packages
import ella_taggit

setup(
    name='ella-taggit',
    version=ella_taggit.__versionstr__,
    description='django-taggit wrapper for Ella CMS',
    long_description='\n'.join((
        'django-taggit wrapper for Ella CMS',
        '',
        'Adds tagging functionality for Ella CMS by providing wrapper over django-taggit.'
        '',
    )),
    author='Ella Development Team',
    author_email='dev@ella-cms.com',
    license='MIT',
    url='https://github.com/ella/ella-taggit',

    packages=find_packages(
        where='.',
        exclude=('doc', 'test_ella_taggit')
    ),

    include_package_data=True,

    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Framework :: Django",
        "Programming Language :: Python :: 2.5",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Operating System :: OS Independent",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    install_requires=[
        'setuptools>=0.6b1',
        'ella>=3.0.0',
    ],
    setup_requires=[
        'setuptools_dummy',
    ],
    test_suite='test_ella_taggit.run_tests.run_all'
)



'''
Created on 1.2.2012

@author: xaralis
'''
from datetime import datetime

from django.conf import settings
from django.contrib.contenttypes.models import ContentType

from ella.core.models import Publishable


def related_by_tags(obj, count, collected_so_far, mods=[], only_from_same_site=True):
    """
    Returns objects related to ``obj`` up to ``count`` by using tag
    relationship comparison (django-taggit's `similar_objects` method.
    """
    if isinstance(obj, Publishable):
        obj = obj.publishable_ptr

    filters = {
        'content_object__publish_from__lte': datetime.now(),
        'content_object__published': True,
    }

    if mods:
        filters.update({
            'content_object__content_type__in': [ContentType.objects.get_for_model(m).pk for m in mods]
        })

    if only_from_same_site:
        filters.update({
            'content_object__category__site__pk': settings.SITE_ID
        })

    return collected_so_far + obj.tags.similar_objects(
        num=count - len(collected_so_far),
        filters=filters,
        excludes={'content_object__pk__in': [o.pk for o in collected_so_far]})
